setwd("C:/Users/pc_156/Dropbox/Research/Forward variable selection for ultra-high dimensional quantile regression/code - submit R2/code - quantileFR/real data analysis")
library(quantreg)
library(rqPen)
library(hqreg)
library(ddpcr)
# library(mvtnorm)
library(mvnfast)
library(hdi)

source("main_functions_data_analysis.r")




###==================
### data manipulation
###==================
### the data set available in R package hdi::riboflavin
data(riboflavin)
y <- riboflavin$y
x <- scale(matrix(riboflavin$x, ncol=ncol(riboflavin$x)), TRUE, TRUE)

corx <- cor(x)[upper.tri(cor(x))]
sum(abs(corx)>0.7)/length(corx)

n <- nrow(x)
p <- ncol(x)
Name <- colnames(riboflavin$x)

###==============================
### Data analysis (select genes)
###==============================


obj.3 <- RDA(x, y, tau=0.3)
obj.5 <- RDA(x, y, tau=0.5)
obj.7 <- RDA(x, y, tau=0.7)


outfun <- function(obj, Name){
  
  list(
    ind.lasso = Name[obj$obj.lasso$ind.hqic],
    ind.alasso = Name[obj$obj.alasso$ind.hqic],
    ind.pq = Name[obj$obj.pq],
    ind.pq.scad = Name[obj$obj.pq.scad$ind.hqic],
    ind.pq.mcp = Name[obj$obj.pq.mcp$ind.hqic],
    ind.sc = Name[obj$obj.sc$ind.act],
    ind.sc.scad = Name[obj$obj.sc.scad$ind.hqic],
    ind.sc.mcp = Name[obj$obj.sc.mcp$ind.hqic],
    ind.gsc = Name[obj$obj.gsc$ind.act],
    ind.gsc.scad = Name[obj$obj.gsc.scad$ind.hqic],
    ind.gsc.mcp = Name[obj$obj.gsc.mcp$ind.hqic],
    ind.gsch = Name[obj$obj.gsch$ind.act],
    ind.gsch.scad = Name[obj$obj.gsch.scad$ind.hqic],
    ind.gsch.mcp = Name[obj$obj.gsch.mcp$ind.hqic],
    ind.fr = Name[obj$obj.fr$ind.act],
    ind.fr.scad = Name[obj$obj.fr.scad$ind.hqic],
    ind.fr.mcp = Name[obj$obj.fr.mcp$ind.hqic], 
    ind.pqfr = Name[obj$obj.pqfr$ind.act[which.min(obj$obj.pqfr$HQIC)]], 
    ind.pqfr.scad = Name[obj$obj.pqfr.scad$ind.hqic], 
    ind.pqfr.mcp = Name[obj$obj.pqfr.mcp$ind.hqic])
}

out.3 <- outfun(obj.3, Name)
out.5 <- outfun(obj.5, Name)
out.7 <- outfun(obj.7, Name)


###==============================
### Evaluate the prediction error
###==============================

###
### prediction performance of Table 4
###
seed.vec <- 1:50
tau.vec <- c(0.3, 0.5, 0.7)

B <- length(seed.vec)
n <- length(y)
p <- ncol(x)
n.tau <- length(tau.vec)
n.train <- 50; n.test <- n - n.train
max.iter <- ceiling(n.train/log(n.train))

c0=log(log(p))/log(p)
xi <- log(log(n.train))

pred.sc  <- pred.gsc  <- pred.fr <- matrix(NA, nrow=B, ncol=n.tau)
pred.sc.lasso <- pred.sc.scad <- pred.sc.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.gsc.lasso <- pred.gsc.scad <- pred.gsc.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.pq.lasso <- pred.pq.scad <- pred.pq.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.pqfr <- pred.pqfr.lasso <- pred.pqfr.scad <- pred.pqfr.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.fr <- pred.fr.lasso <- pred.fr.scad <- pred.fr.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.lasso <- pred.alasso <- matrix(NA, nrow=B, ncol=n.tau)
pred.h <- pred.h.lasso <- pred.h.mcp <- pred.h.scad <- matrix(NA, nrow=B, ncol=n.tau)

nz.sc <- nz.sc.lasso <- nz.sc.scad <- nz.sc.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.gsc <- nz.gsc.lasso <- nz.gsc.scad <- nz.gsc.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.pq.lasso <- nz.pq.scad <- nz.pq.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.pqfr <- nz.pqfr.lasso <- nz.pqfr.scad <- nz.pqfr.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.fr <- nz.fr.lasso <- nz.fr.scad <- nz.fr.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.lasso <- nz.alasso <- matrix(NA, nrow=B, ncol=n.tau)
nz.h  <- nz.h.lasso <- nz.h.mcp <- nz.h.scad <- matrix(NA, nrow=B, ncol=n.tau)



eval.pred <- function(beta.est, x.test, y.test, tau){
  mean(check(y.test - cbind(1,as.matrix(x.test))%*%beta.est, tau))
}



B0.vec <- 1:50
for(b in B0.vec){
  
  set.seed(seed.vec[b])
  
  ind.train <- sort(sample(1:n, n.train, replace=FALSE), decreasing=FALSE)
  ind.test <- setdiff(1:n, ind.train)
  x.train <- as.matrix(x[ind.train,]); x.test <- as.matrix(x[ind.test,])
  y.train <- y[ind.train]; y.test <- y[ind.test]
  
  for(i in 1:length(tau.vec)){
    
    tau <- tau.vec[i]
    
    
    # ### SC
    # obj.sc <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="SC", stop.method="LIC", xi=xi, method="br")
    # beta.sc <- obj.sc$beta[, ncol(obj.sc$beta)]
    # pred.sc[b,i] <- eval.pred(beta.sc, x.test, y.test, tau)
    # nz.sc[b,i] <- sum(beta.sc!=0)-1
    # 
    # ### SC+Lasso
    # beta.sc.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=obj.sc$ind.act, c0=c0)$beta.hqic
    # pred.sc.lasso[b,i] <- eval.pred(beta.sc.lasso, x.test, y.test, tau)
    # nz.sc.lasso[b,i] <- sum(beta.sc.lasso!=0)-1
    # 
    # ### SC+scad
    # beta.sc.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=obj.sc$ind.act, c0=c0)$beta.hqic
    # pred.sc.scad[b,i] <- eval.pred(beta.sc.scad, x.test, y.test, tau)
    # nz.sc.scad[b,i] <- sum(beta.sc.scad!=0)-1
    # 
    # ### SC+mcp
    # beta.sc.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=obj.sc$ind.act, c0=c0)$beta.hqic
    # pred.sc.mcp[b,i] <- eval.pred(beta.sc.mcp, x.test, y.test, tau)
    # nz.sc.mcp[b,i] <- sum(beta.sc.mcp!=0)-1
    # 
    # 
    # ### gSC
    # m <- ceiling(n.train/log(n.train))
    # obj.gsc <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="SC+FR", stop.method="LIC", xi=xi, method="br", m=m)
    # beta.gsc <- obj.gsc$beta[, ncol(obj.gsc$beta)]
    # pred.gsc[b,i] <- eval.pred(beta.gsc, x.test, y.test, tau)
    # nz.gsc[b,i] <- sum(beta.gsc!=0)-1
    # 
    # ### gSC+lasso
    # beta.gsc.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=obj.gsc$ind.act, c0=c0)$beta.hqic
    # pred.gsc.lasso[b,i] <- eval.pred(beta.gsc.lasso, x.test, y.test, tau)
    # nz.gsc.lasso[b,i] <- sum(beta.gsc.lasso!=0)-1
    # 
    # ### gSC+scad
    # beta.gsc.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=obj.gsc$ind.act, c0=c0)$beta.hqic
    # pred.gsc.scad[b,i] <- eval.pred(beta.gsc.scad, x.test, y.test, tau)
    # nz.gsc.scad[b,i] <- sum(beta.gsc.scad!=0)-1
    # 
    # ### gSC+mcp
    # beta.gsc.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=obj.gsc$ind.act, c0=c0)$beta.hqic
    # pred.gsc.mcp[b,i] <- eval.pred(beta.gsc.mcp, x.test, y.test, tau)
    # nz.gsc.mcp[b,i] <- sum(beta.gsc.mcp!=0)-1
    # 
    # 
    # ### fr
    # obj.fr <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="FR", stop.method="LIC", xi=xi, method="br")
    # beta.fr <- obj.fr$beta[, ncol(obj.fr$beta)]
    # pred.fr[b,i] <- eval.pred(beta.fr, x.test, y.test, tau)
    # nz.fr[b,i] <- sum(beta.fr!=0)-1
    # 
    # ### fr+lasso
    # beta.fr.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=obj.fr$ind.act, c0=c0)$beta.hqic
    # pred.fr.lasso[b,i] <- eval.pred(beta.fr.lasso, x.test, y.test, tau)
    # nz.fr.lasso[b,i] <- sum(beta.fr.lasso!=0)-1
    # 
    # ### fr+scad
    # beta.fr.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=obj.fr$ind.act, c0=c0)$beta.hqic
    # pred.fr.scad[b,i] <- eval.pred(beta.fr.scad, x.test, y.test, tau)
    # nz.fr.scad[b,i] <- sum(beta.fr.scad!=0)-1
    # 
    # ### fr+mcp
    # beta.fr.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=obj.fr$ind.act, c0=c0)$beta.hqic
    # pred.fr.mcp[b,i] <- eval.pred(beta.fr.mcp, x.test, y.test, tau)
    # nz.fr.mcp[b,i] <- sum(beta.fr.mcp!=0)-1
    
    ### H
    obj.h <- rq.greedy.h(x=x.train, y=y.train, tau=tau, stop.method="LIC", xi=xi, method="br")
    beta.h <- obj.h$beta
    pred.h[b,i] <- eval.pred(beta.h, x.test, y.test, tau)
    nz.h[b,i] <- sum(beta.h!=0)-1
    
    ### H+lasso
    beta.h.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=obj.h$ind.act, c0=c0)$beta.hqic
    pred.h.lasso[b,i] <- eval.pred(beta.h.lasso, x.test, y.test, tau)
    nz.h.lasso[b,i] <- sum(beta.h.lasso!=0)-1
    
    ### h+scad
    beta.h.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=obj.h$ind.act, c0=c0)$beta.hqic
    pred.h.scad[b,i] <- eval.pred(beta.h.scad, x.test, y.test, tau)
    nz.h.scad[b,i] <- sum(beta.h.scad!=0)-1
    
    ### h+mcp
    beta.h.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=obj.h$ind.act, c0=c0)$beta.hqic
    pred.h.mcp[b,i] <- eval.pred(beta.h.mcp, x.test, y.test, tau)
    nz.h.mcp[b,i] <- sum(beta.h.mcp!=0)-1
    
    
    

    # ### PQ
    # ind.pq <- which(Screen_Mar(x.train, y.train, tt=tau, max.iter)==1)
    # 
    # ### PQ+Lasso
    # beta.pq.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=ind.pq, c0=c0)$beta.hqic
    # pred.pq.lasso[b,i] <- eval.pred(beta.pq.lasso, x.test, y.test, tau)
    # nz.pq.lasso[b,i] <- sum(beta.pq.lasso!=0)-1
    # 
    # ### PQ+SCAD
    # beta.pq.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=ind.pq, c0=c0)$beta.hqic
    # pred.pq.scad[b,i] <- eval.pred(beta.pq.scad, x.test, y.test, tau)
    # nz.pq.scad[b,i] <- sum(beta.pq.scad!=0)-1
    # 
    # ### PQ+mcp
    # beta.pq.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=ind.pq, c0=c0)$beta.hqic
    # pred.pq.mcp[b,i] <- eval.pred(beta.pq.mcp, x.test, y.test, tau)
    # nz.pq.mcp[b,i] <- sum(beta.pq.mcp!=0)-1
    # 
    # 
    # ### pqfr
    # obj.pqfr <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="PQU.fr", method="br", c0=1)
    # beta.pqfr <- obj.pqfr$beta[, which.min(obj.pqfr$HQIC)]
    # pred.pqfr[b,i] <- eval.pred(beta.pqfr, x.test, y.test, tau)
    # ind.pqfr <- obj.pqfr$ind.act[1:which.min(obj.pqfr$HQIC)]
    # nz.pqfr[b,i] <- length(ind.pqfr)
    # 
    # ### pqfr + Lasso
    # beta.pqfr.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=ind.pqfr, c0=c0)$beta.hqic
    # pred.pqfr.lasso[b,i] <- eval.pred(beta.pqfr.lasso, x.test, y.test, tau)
    # nz.pqfr.lasso[b,i] <- sum(beta.pqfr.lasso!=0)-1
    # 
    # ### pqfr + SCAD
    # beta.pqfr.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=ind.pqfr, c0=c0)$beta.hqic
    # pred.pqfr.scad[b,i] <- eval.pred(beta.pqfr.scad, x.test, y.test, tau)
    # nz.pqfr.scad[b,i] <- sum(beta.pqfr.scad!=0)-1
    # 
    # ### pqfr + mcp
    # beta.pqfr.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=ind.pqfr, c0=c0)$beta.hqic
    # pred.pqfr.mcp[b,i] <- eval.pred(beta.pqfr.mcp, x.test, y.test, tau)
    # nz.pqfr.mcp[b,i] <- sum(beta.pqfr.mcp!=0)-1
    # 
    # 
    # 
    # ### Lasso
    # obj.lasso <- QICD.pen2(x.train, y.train, tau=tau, penalty="LASSO", c0=1/log(n.train)/log(p))
    # beta.lasso <- obj.lasso$beta.hqic
    # pred.lasso[b,i] <- eval.pred(beta.lasso, x.test, y.test, tau.vec[i])
    # nz.lasso[b,i] <- sum(beta.lasso!=0)-1
    # 
    # ### Lasso + ALasso
    # if(length(obj.lasso$ind.hqic)>0){
    #   pf <- 1/(abs(beta.lasso[-1])+1e-5)
    #   obj.alasso <- ALASSO(x.train, y.train, ind=NULL, method="quantile", tau=tau, penalty.factor=pf, c0=c0)
    #   pred.alasso[b,i] <- eval.pred(obj.alasso$beta.hqic, x.test, y.test, tau.vec[i])
    #   nz.alasso[b,i] <- sum(obj.alasso$beta.hqic!=0)-1
    # } else {
    #   pred.alasso[b,i] <- pred.lasso[b,i]
    #   nz.alasso[b,i] <- 0
    # }


  }
  
  cat(b, '/')
}

name <- c(
  "CQU_LASSO", "CQU_SCAD", "CQU_MCP",
  "SC", "SC_LASSO", "SC_SCAD", "SC_MCP",
  "gSC", "gSC_LASSO", "gSC_SCAD", "gSC_MCP",
  "FR", "FR_LASSO", "FR_SCAD", "FR_MCP",
  "H", "H_LASSO", "H_SCAD", "H_MCP",
  "FR-PQU+QBIC", "FR-PQU+QBIC_LASSO", "FR-PQU+QBIC_SCAD", "FR-PQU+QBIC_MCP",
  "LASSO", "ALASSO"
)

m0 <- 3

out.med <- round(
rbind(c(median(pred.pq.lasso[,m0]), median(nz.pq.lasso)), 
      c(median(pred.pq.scad[,m0]), median(nz.pq.scad)), 
      c(median(pred.pq.mcp[,m0]), median(nz.pq.mcp)), 
      c(median(pred.sc[,m0]), median(nz.sc)), 
      c(median(pred.sc.lasso[,m0]), median(nz.sc.lasso)), 
      c(median(pred.sc.scad[,m0]), median(nz.sc.scad)), 
      c(median(pred.sc.mcp[,m0]), median(nz.sc.mcp)),
      c(median(pred.gsc[,m0]), median(nz.gsc)), 
      c(median(pred.gsc.lasso[,m0]), median(nz.gsc.lasso)), 
      c(median(pred.gsc.scad[,m0]), median(nz.gsc.scad)), 
      c(median(pred.gsc.mcp[,m0]), median(nz.gsc.mcp)),
      c(median(pred.fr[,m0]), median(nz.fr)), 
      c(median(pred.fr.lasso[,m0]), median(nz.fr.lasso)), 
      c(median(pred.fr.scad[,m0]), median(nz.fr.scad)), 
      c(median(pred.fr.mcp[,m0]), median(nz.fr.mcp)),
      c(median(pred.h[,m0]), median(nz.h)), 
      c(median(pred.h.lasso[,m0]), median(nz.h.lasso)), 
      c(median(pred.h.scad[,m0]), median(nz.h.scad)), 
      c(median(pred.h.mcp[,m0]), median(nz.h.mcp)),
      c(median(pred.pqfr[,m0]), median(nz.pqfr)), 
      c(median(pred.pqfr.lasso[,m0]), median(nz.pqfr.lasso)), 
      c(median(pred.pqfr.scad[,m0]), median(nz.pqfr.scad)), 
      c(median(pred.pqfr.mcp[,m0]), median(nz.pqfr.mcp)),

      c(median(pred.lasso[,m0]), median(nz.lasso)),
      c(median(pred.alasso[,m0]), median(nz.alasso))
      ), 3)

out.sd <- round(
  rbind(c(sd(pred.pq.lasso[,m0]), sd(nz.pq.lasso)), 
        c(sd(pred.pq.scad[,m0]), sd(nz.pq.scad)), 
        c(sd(pred.pq.mcp[,m0]), sd(nz.pq.mcp)), 
        c(sd(pred.sc[,m0]), sd(nz.sc)), 
        c(sd(pred.sc.lasso[,m0]), sd(nz.sc.lasso)), 
        c(sd(pred.sc.scad[,m0]), sd(nz.sc.scad)), 
        c(sd(pred.sc.mcp[,m0]), sd(nz.sc.mcp)),
        c(sd(pred.gsc[,m0]), sd(nz.gsc)), 
        c(sd(pred.gsc.lasso[,m0]), sd(nz.gsc.lasso)), 
        c(sd(pred.gsc.scad[,m0]), sd(nz.gsc.scad)), 
        c(sd(pred.gsc.mcp[,m0]), sd(nz.gsc.mcp)),
        c(sd(pred.fr[,m0]), sd(nz.fr)), 
        c(sd(pred.fr.lasso[,m0]), sd(nz.fr.lasso)), 
        c(sd(pred.fr.scad[,m0]), sd(nz.fr.scad)), 
        c(sd(pred.fr.mcp[,m0]), sd(nz.fr.mcp)),
        c(sd(pred.h[,m0]), sd(nz.h)), 
        c(sd(pred.h.lasso[,m0]), sd(nz.h.lasso)), 
        c(sd(pred.h.scad[,m0]), sd(nz.h.scad)), 
        c(sd(pred.h.mcp[,m0]), sd(nz.h.mcp)),
        c(sd(pred.pqfr[,m0]), sd(nz.pqfr)), 
        c(sd(pred.pqfr.lasso[,m0]), sd(nz.pqfr.lasso)), 
        c(sd(pred.pqfr.scad[,m0]), sd(nz.pqfr.scad)), 
        c(sd(pred.pqfr.mcp[,m0]), sd(nz.pqfr.mcp)),

        c(sd(pred.lasso[,m0]), sd(nz.lasso)),
        c(sd(pred.alasso[,m0]), sd(nz.alasso))
  ), 3)

rownames(out.med) <- name

out.med
out.sd







###
### prediction performance of Table 9
###
pred.scf <- pred.scf.lasso <- pred.scf.scad <- pred.scf.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.gscf <- pred.gscf.lasso <- pred.gscf.scad <- pred.gscf.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.frf <- pred.frf.lasso <- pred.frf.scad <- pred.frf.mcp <- matrix(NA, nrow=B, ncol=n.tau)
pred.pqfrf <- pred.pqfrf.lasso <- pred.pqfrf.scad <- pred.pqfrf.mcp <- matrix(NA, nrow=B, ncol=n.tau)

nz.scf <- nz.scf.lasso <- nz.scf.scad <- nz.scf.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.gscf <- nz.gscf.lasso <- nz.gscf.scad <- nz.gscf.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.frf <- nz.frf.lasso <- nz.frf.scad <- nz.frf.mcp <- matrix(NA, nrow=B, ncol=n.tau)
nz.pqfrf <- nz.pqfrf.lasso <- nz.pqfrf.scad <- nz.pqfrf.mcp <- matrix(NA, nrow=B, ncol=n.tau)



B0.vec <- 1:50
for(b in B0.vec){
  
  set.seed(seed.vec[b])
  
  ind.train <- sort(sample(1:n, n.train, replace=FALSE), decreasing=FALSE)
  ind.test <- setdiff(1:n, ind.train)
  x.train <- as.matrix(x[ind.train,]); x.test <- as.matrix(x[ind.test,])
  y.train <- y[ind.train]; y.test <- y[ind.test]
  
  for(i in 1:length(tau.vec)){
    
    tau <- tau.vec[i]
    
    
    ### SCf
    obj.scf <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="SC", stop.method="LIC", xi=xi, method="br", K=30)
    beta.scf <- obj.scf$beta[, ncol(obj.scf$beta)]
    pred.scf[b,i] <- eval.pred(beta.scf, x.test, y.test, tau)
    nz.scf[b,i] <- length(obj.scf$ind.act)
    
    ### SCf+Lasso
    beta.scf.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=obj.scf$ind.act, c0=c0)$beta.hqic
    pred.scf.lasso[b,i] <- eval.pred(beta.scf.lasso, x.test, y.test, tau)
    nz.scf.lasso[b,i] <- sum(beta.scf.lasso!=0)-1
    
    ### SC+scad
    beta.scf.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=obj.scf$ind.act, c0=c0)$beta.hqic
    pred.scf.scad[b,i] <- eval.pred(beta.scf.scad, x.test, y.test, tau)
    nz.scf.scad[b,i] <- sum(beta.scf.scad!=0)-1
    
    ### SC+mcp
    beta.scf.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=obj.scf$ind.act, c0=c0)$beta.hqic
    pred.scf.mcp[b,i] <- eval.pred(beta.scf.mcp, x.test, y.test, tau)
    nz.scf.mcp[b,i] <- sum(beta.scf.mcp!=0)-1
    
    
    ### gSCf
    m <- ceiling(n.train/log(n.train))
    obj.gscf <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="SC+FR", stop.method="LIC", xi=xi, method="br", m=m, K=30)
    beta.gscf <- obj.gscf$beta[, ncol(obj.gscf$beta)]
    pred.gscf[b,i] <- eval.pred(beta.gscf, x.test, y.test, tau)
    nz.gscf[b,i] <- length(obj.gscf$ind.act)

    ### gSCf+lasso
    beta.gscf.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=obj.gscf$ind.act, c0=c0)$beta.hqic
    pred.gscf.lasso[b,i] <- eval.pred(beta.gscf.lasso, x.test, y.test, tau)
    nz.gscf.lasso[b,i] <- sum(beta.gscf.lasso!=0)-1

    ### gSCf+scad
    beta.gscf.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=obj.gscf$ind.act, c0=c0)$beta.hqic
    pred.gscf.scad[b,i] <- eval.pred(beta.gscf.scad, x.test, y.test, tau)
    nz.gscf.scad[b,i] <- sum(beta.gscf.scad!=0)-1

    ### gSCf+mcp
    beta.gscf.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=obj.gscf$ind.act, c0=c0)$beta.hqic
    pred.gscf.mcp[b,i] <- eval.pred(beta.gscf.mcp, x.test, y.test, tau)
    nz.gscf.mcp[b,i] <- sum(beta.gscf.mcp!=0)-1


    ### frf
    obj.frf <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="FR", stop.method="LIC", xi=xi, method="br", K=30)
    beta.frf <- obj.frf$beta[, ncol(obj.frf$beta)]
    pred.frf[b,i] <- eval.pred(beta.frf, x.test, y.test, tau)
    nz.frf[b,i] <- length(obj.frf$ind.act)

    ### frf+lasso
    beta.frf.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=obj.frf$ind.act, c0=c0)$beta.hqic
    pred.frf.lasso[b,i] <- eval.pred(beta.frf.lasso, x.test, y.test, tau)
    nz.frf.lasso[b,i] <- sum(beta.frf.lasso!=0)-1

    ### frf+scad
    beta.frf.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=obj.frf$ind.act, c0=c0)$beta.hqic
    pred.frf.scad[b,i] <- eval.pred(beta.frf.scad, x.test, y.test, tau)
    nz.frf.scad[b,i] <- sum(beta.frf.scad!=0)-1

    ### fr+mcp
    beta.frf.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=obj.frf$ind.act, c0=c0)$beta.hqic
    pred.frf.mcp[b,i] <- eval.pred(beta.frf.mcp, x.test, y.test, tau)
    nz.frf.mcp[b,i] <- sum(beta.frf.mcp!=0)-1
    
    
    obj.pqfr <- rq.greedy(x=x.train, y=y.train, tau=tau, screen.method="PQU.fr", method="br", c0=1)
    
    ### pqfrf
    beta.pqfrf <- obj.pqfr$beta[, ncol(obj.pqfr$beta)]
    pred.pqfrf[b,i] <- eval.pred(beta.pqfrf, x.test, y.test, tau)
    ind.pqfrf <- obj.pqfr$ind.act
    nz.pqfrf[b,i] <- length(ind.pqfrf)
    
    ### pqfrf + Lasso
    beta.pqfrf.lasso <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="LASSO", ind=ind.pqfrf, c0=c0)$beta.hqic
    pred.pqfrf.lasso[b,i] <- eval.pred(beta.pqfrf.lasso, x.test, y.test, tau)
    nz.pqfrf.lasso[b,i] <- sum(beta.pqfrf.lasso!=0)-1
    
    ### pqfrf + SCAD
    beta.pqfrf.scad <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="SCAD", ind=ind.pqfrf, c0=c0)$beta.hqic
    pred.pqfrf.scad[b,i] <- eval.pred(beta.pqfrf.scad, x.test, y.test, tau)
    nz.pqfrf.scad[b,i] <- sum(beta.pqfrf.scad!=0)-1
    
    ### pqfrf + mcp
    beta.pqfrf.mcp <- QICD.pen2(x=x.train, y=y.train, tau=tau, penalty="MCP", ind=ind.pqfrf, c0=c0)$beta.hqic
    pred.pqfrf.mcp[b,i] <- eval.pred(beta.pqfrf.mcp, x.test, y.test, tau)
    nz.pqfrf.mcp[b,i] <- sum(beta.pqfrf.mcp!=0)-1
    
  }
  
  cat(b, '/')
}


name <- c(
  "SC", "SC_LASSO", "SC_SCAD", "SC_MCP",
  "gSC", "gSC_LASSO", "gSC_SCAD", "gSC_MCP",
  "FR", "FR_LASSO", "FR_SCAD", "FR_MCP",
  "FR-PQU", "FR-PQU_LASSO", "FR-PQU_SCAD", "FR-PQU_MCP"
)

m0 <- 3

out.med <- round(
  rbind(
        c(median(pred.scf[,m0]), median(nz.scf)), 
        c(median(pred.scf.lasso[,m0]), median(nz.scf.lasso)), 
        c(median(pred.scf.scad[,m0]), median(nz.scf.scad)), 
        c(median(pred.scf.mcp[,m0]), median(nz.scf.mcp)),
        c(median(pred.gscf[,m0]), median(nz.gscf)), 
        c(median(pred.gscf.lasso[,m0]), median(nz.gscf.lasso)), 
        c(median(pred.gscf.scad[,m0]), median(nz.gscf.scad)), 
        c(median(pred.gscf.mcp[,m0]), median(nz.gscf.mcp)),
        c(median(pred.frf[,m0]), median(nz.frf)), 
        c(median(pred.frf.lasso[,m0]), median(nz.frf.lasso)), 
        c(median(pred.frf.scad[,m0]), median(nz.frf.scad)), 
        c(median(pred.frf.mcp[,m0]), median(nz.frf.mcp)),
        c(median(pred.pqfrf[,m0]), median(nz.pqfrf)), 
        c(median(pred.pqfrf.lasso[,m0]), median(nz.pqfrf.lasso)), 
        c(median(pred.pqfrf.scad[,m0]), median(nz.pqfrf.scad)), 
        c(median(pred.pqfrf.mcp[,m0]), median(nz.pqfrf.mcp))
  ), 3)

out.sd <- round(
  rbind(
        c(sd(pred.scf[,m0]), sd(nz.scf)), 
        c(sd(pred.scf.lasso[,m0]), sd(nz.scf.lasso)), 
        c(sd(pred.scf.scad[,m0]), sd(nz.scf.scad)), 
        c(sd(pred.scf.mcp[,m0]), sd(nz.scf.mcp)),
        c(sd(pred.gscf[,m0]), sd(nz.gscf)), 
        c(sd(pred.gscf.lasso[,m0]), sd(nz.gscf.lasso)), 
        c(sd(pred.gscf.scad[,m0]), sd(nz.gscf.scad)), 
        c(sd(pred.gscf.mcp[,m0]), sd(nz.gscf.mcp)),
        c(sd(pred.frf[,m0]), sd(nz.frf)), 
        c(sd(pred.frf.lasso[,m0]), sd(nz.frf.lasso)), 
        c(sd(pred.frf.scad[,m0]), sd(nz.frf.scad)), 
        c(sd(pred.frf.mcp[,m0]), sd(nz.frf.mcp)),
        c(sd(pred.pqfrf[,m0]), sd(nz.pqfrf)), 
        c(sd(pred.pqfrf.lasso[,m0]), sd(nz.pqfrf.lasso)), 
        c(sd(pred.pqfrf.scad[,m0]), sd(nz.pqfrf.scad)), 
        c(sd(pred.pqfrf.mcp[,m0]), sd(nz.pqfrf.mcp))
  ), 3)

rownames(out.med) <- name

out.med
out.sd